--[[=============================================================================
    Commands received from the Carrier Thermostat drivers (ReceivedFromProxy)

    Copyright 2015 Control4 Corporation. All Rights Reserved.
===============================================================================]]

-- This macro is utilized to identify the version string of the driver template version used.
if (TEMPLATE_VERSION ~= nil) then
	TEMPLATE_VERSION.proxy_commands = "2014.10.31"
end

PROXY_ID = 2

-- Commands received from carrier_infinity_thermostat drivers
function PRX_CMD.ADD_DRIVER(bindingID, tParams)
	local system = tParams["SYSTEM"]
	local zone = tParams["ZONE"]
	local deviceId = tParams["DEVICE_ID"]

	gDeviceList[deviceId] = {}
	gDeviceList[deviceId].system = system
	gDeviceList[deviceId].zone = zone

	if (gCon ~= nil and gCon._IsConnected) then
		GetDeviceSettings(system, zone)
		GetDeviceSchedule(system, zone)
	end
end

function PRX_CMD.REMOVE_DRIVER(bindingID, tParams)
	local deviceId = tParams["DEVICE_ID"]
	
	gDeviceList[deviceId] = nil
end

function PRX_CMD.CURRENT_SETTINGS(bindingID, tParams)
	GetSystemSettings(tParams)
	GetDeviceSettings(system, zone)
	GetDeviceSchedule(system, zone)
end

function PRX_CMD.SET_MODE(bindingID, tParams)
	local system = tParams["SYSTEM"]
	local hvacMode = tParams["HVACMODE"]
	
	QueuePriority1Command(string.format("S%dMODE!%s", system, hvacMode))	-- set mode
	QueuePriority1Command(string.format("S%dMODE?", system))				-- query current mode
end

function PRX_CMD.SET_FAN_MODE(bindingID, tParams)
	local system = tParams["SYSTEM"]
	local zone = tParams["ZONE"]
	local fanMode = tParams["FANMODE"]
	
	if ("medium" == string.lower(fanMode)) then
		fanMode = "MED"
	end

	QueuePriority1Command(string.format("S%dZ%dFAN!%s", system, zone, fanMode))	-- set fan mode
	QueuePriority1Command(string.format("S%dZ%dFAN?", system, zone))				-- query current fan mode
end

function PRX_CMD.SET_HEAT_SETPOINT(bindingID, tParams)
	local system = tParams["SYSTEM"]
	local zone = tParams["ZONE"]

	QueuePriority1Command(string.format("S%dZ%dHTSP!%s", system, zone, tParams["SETPOINT"]))	-- set heat setpoint
	QueuePriority1Command(string.format("S%dZ%dHTSP?", system, zone))						-- query current heat setpoint
end

function PRX_CMD.SET_COOL_SETPOINT(bindingID, tParams)
	local system = tParams["SYSTEM"]
	local zone = tParams["ZONE"]

	QueuePriority1Command(string.format("S%dZ%dCLSP!%s", system, zone, tParams["SETPOINT"]))	-- set cool setpoint
	QueuePriority1Command(string.format("S%dZ%dCLSP?", system, zone))						-- query current cool setpoint
end

function PRX_CMD.SET_SCALE(bindingID, tParams)
	local system = tParams["SYSTEM"]
	local scale = tParams["SCALE"]
	
	if (string.sub(scale, 1, 1) == "F") then
		scale = "E"
	else
		scale = "M"
	end
	
	QueuePriority1Command(string.format("S%dCFGEM!%s", system, scale))	-- set scale
	C4:SendToProxy(PROXY_ID, "SCALE_CHANGED", tParams)			-- 
	QueuePriority1Command(string.format("S%dCFGEM?", system))			-- query current scale
end

function PRX_CMD.SET_HOLD_MODE(bindingID, tParams)
	local system = tParams["SYSTEM"]
	local zone = tParams["ZONE"]
	local holdMode = tParams["HOLDMODE"]
	local cmd

	if (holdMode == "OFF") then
		cmd = string.format("S%dZ%dHOLD!OFF", system, zone)
	elseif (holdMode == "PERMANENT") then
		cmd = string.format("S%dZ%dHOLD!ON", system, zone)
	elseif (holdMode == "2_HOURS") then
		cmd = string.format("S%dZ%dOTMR!02:00", system, zone)
	elseif (holdMode == "TO_NEXT") then
		LogInfo("TO_NEXT command not supported")
		return
	end

	QueuePriority1Command(cmd)											-- set hold mode
	QueuePriority1Command(string.format("S%dZ%dHOLD?", system, zone))	-- query current hold mode
end

function PRX_CMD.SET_VACATION_MODE(bindingID, tParams)
	local system = tParams["SYSTEM"]
	
	QueuePriority1Command(string.format("S%dVACDayS!%03d", system, tParams["NUM_DAYS"]))	-- set number vacation days
	QueuePriority1Command(string.format("S%dVACDAYS?", system))							-- query current number vacation days
	QueuePriority1Command(string.format("S%dVACAT?", system))							-- query current vacation state
end

function PRX_CMD.SET_LEGACY_SCHEDULE_ENTRY(bindingID, tParams)
	local system = tParams["SYSTEM"]
	local zone = tParams["ZONE"]
	local minutes = tonumber(tParams["ENTRY_TIME"])
	local timePeriod = "A"

	if (minutes >= 720) then
		timePeriod = "P"
		minutes = minutes - 720
	end
	
	local hour = math.floor(minutes/60)
	local minute = minutes - (hour * 60)

	-- set schedule entry		
	local cmd = string.format("S%dZ%dPGM%s%s!%02d:%02d%s,%d,%d,AUTO", system, zone, DAY_INDEX_REV[tonumber(tParams["DAY_INDEX"])],
			ENTRY_INDEX_REV[tonumber(tParams["ENTRY_INDEX"])], hour, minute, timePeriod, tParams["HEAT_SETPOINT"], tParams["COOL_SETPOINT"])
	QueuePriority1Command(cmd)
	-- Query schedule entry
	cmd = string.format("S%dZ%dPGM%s%s?", system, zone, DAY_INDEX_REV[tonumber(tParams["DAY_INDEX"])], ENTRY_INDEX_REV[tonumber(tParams["ENTRY_INDEX"])])
	QueuePriority1Command(cmd)
end

